#!/usr/bin/perl

use strict;
use warnings;
use lib "/usr/local/esia/plugins"  ;
use vars qw($PROGNAME);
use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;

my $TIMEOUT = 50;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

$PROGNAME = "check_snmp_pdu_health.pl";
sub print_help ();
sub print_usage ();

my $version='1.0';
my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 1;
my $voltage = 220;
my ($hostname);

use Getopt::Long;
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"U=i" => \$voltage,"voltage=i" => \$voltage,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
# -V & --version print version
if ($opt_V) { print_revision($PROGNAME,'$Revision: 0.1 $ '); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2) {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community
	);
} elsif ($snmp_version == 3) {
	my ($v3_username,$v3_password,$v3_protocol,$v3_priv_passphrase,$v3_priv_protocol) = split(":",$community);
	my @auth = ();
	if (defined($v3_password)) { push(@auth,($v3_password =~ /^0x/) ? 'authkey' : 'authpassword',$v3_password); }
	if (defined($v3_protocol)) { push(@auth,'authprotocol',$v3_protocol); }
	if (defined($v3_priv_passphrase)) { push(@auth,($v3_priv_passphrase =~ /^0x/) ? 'privkey' : 'privpassword',$v3_priv_passphrase); }
	if (defined($v3_priv_protocol)) { push(@auth,'privprotocol',$v3_priv_protocol); }

	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv3',
		-username => $v3_username,
		@auth
	);
} else {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community
	);
}

if (!defined($snmp)) {
	print ("UNKNOWN - SNMP error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "";

my $oid_pduloadstatus="1.3.6.1.4.1.318.1.1.26.4.3.1.4.1";
my $oid_pduhundredthskw="1.3.6.1.4.1.318.1.1.26.4.3.1.5.1";
my $oid_pdutenthskwh="1.3.6.1.4.1.318.1.1.26.4.3.1.9.1";
#affichage pour les status
my %pduLoadStatuses = (
	1 => "lowLoad(1)",
	2 => "normal(2)",
	3 => "nearOverload(3)",
	4 => "overload(4)"
);

my @snmpoids;
push (@snmpoids,$oid_pduloadstatus);
push (@snmpoids,$oid_pduhundredthskw);
push (@snmpoids,$oid_pdutenthskwh);

my $result = $snmp->get_request(
	-varbindlist => \@snmpoids
);

if (!defined($result)) {
	sleep(1);
	$result = $snmp->get_request(
		-varbindlist => \@snmpoids
	);
}

if (!defined($result)) {
	sleep(1);
	$result = $snmp->get_request(
		-varbindlist => \@snmpoids
	);
}

if (!defined($result)) {
	my $answer = $snmp->error;
	$snmp->close;
	print ("UNKNOWN - SNMP error: $answer\n");
	exit $ERRORS{'UNKNOWN'};
}

$oid_pduloadstatus = $result->{$oid_pduloadstatus};
$oid_pduhundredthskw = $result->{$oid_pduhundredthskw};
$oid_pdutenthskwh = $result->{$oid_pdutenthskwh};

if (($oid_pduloadstatus !~ /^noSuch/)) 
{
	$output = "PDU status is ".$pduLoadStatuses{$oid_pduloadstatus}."\n";
	if($oid_pduloadstatus == 4)
	{
		$state = 'CRITICAL';
	}
	elsif($oid_pduloadstatus == 1 || $oid_pduloadstatus == 3)
	{
		$state = 'WARNING';
	}
	else
	{
		$state = 'OK';	
	}

	#Consommation KW
	if (($oid_pduhundredthskw !~ /^noSuch/)) 
	{
		my $watts = ($oid_pduhundredthskw*0.01)*1000;
		my $amperes = sprintf("%.2f",$watts/$voltage);
		$output .= "The power consumption of the Rack PDU load is $amperes A (= $watts Watts)\n";
	}
	#Consommation KW/H
	if (($oid_pdutenthskwh !~ /^noSuch/)) 
	{
		my $KWH = $oid_pdutenthskwh*0.1;
		$output .= "The Rack PDU load energy consumption is $KWH kWh\n";
	}

}
else
{
	$state = "UNKNOWN";
	$output = "PDU status not available\n";
}

print "$state : $output";
exit $ERRORS{$state};

sub print_help() {
	print "Check PDU status via SNMP.\n";
	print "Version : ".$version."\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community or v3 auth (defaults to public)\n";
	print "                     (v3 specified as username:authpassword:... )\n";
	print "                       username = SNMPv3 security name\n";
	print "                       authpassword = SNMPv3 authentication pass phrase (or hexidecimal key)\n";
	print "                       authprotocol = SNMPv3 authentication protocol (md5 (default) or sha)\n";
	print "                       privpassword = SNMPv3 privacy pass phrase (or hexidecmal key)\n";
	print "                       privprotocol = SNMPv3 privacy protocol (des (default) or aes)\n";
	print " - U (voltage)		Specify the voltage (U) (default : 220V)\n";
	print " -v (--snmp_version) 1 for SNMP v1 (default)\n";
	print "                     2 for SNMP v2c\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

