#!/usr/bin/perl -w 
############################## check_snmp_load #################
my $Version='1.0';
# Date : Oct 20 2014
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib '/usr/local/gesa/plugins/lib';
use GesaProtocol;


my $PROGNAME='get_snmp_interface.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 2;
my $type='pc';
my $stat=0;
my ($hostname);
my ($snmp_seclvl,$snmp_user,$snmp_auth_prot,$snmp_auth_key,$snmp_priv_prot,$snmp_priv_key);
sub par_num { return $a <=> $b }
sub print_help() {
	printf "get SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community or v3 auth (defaults to public)\n";
	print "                     (v3 specified as username:authpassword:... )\n";
	print "                       username = SNMPv3 security name\n";
	print "                       authpassword = SNMPv3 authentication pass phrase (or hexidecimal key)\n";
	print "                       authprotocol = SNMPv3 authentication protocol (md5 (default) or sha)\n";
	print "                       privpassword = SNMPv3 privacy pass phrase (or hexidecmal key)\n";
	print "                       privprotocol = SNMPv3 privacy protocol (des (default) or aes)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print "                     3 for SNMP v3\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"T=s" => \$type,	"type=s" => \$type,
	"s=i" => \$stat,	"stat=i"=>\$stat,
	#SNMPV3
        "L=s" => \$snmp_seclvl, "snmp_seclvl=s" => \$snmp_seclvl,
        "u=s" => \$snmp_user,   "snmp_user=s" => \$snmp_user,
        "a=s" => \$snmp_auth_prot, "snmp_auth_prot=s" => \$snmp_auth_prot,
        "A=s" => \$snmp_auth_key, "snmp_auth_key=s" => \$snmp_auth_key,
        "x=s" => \$snmp_priv_prot, "snmp_priv_prot=s" => \$snmp_priv_prot,
        "X=s" => \$snmp_priv_key, "snmp_priv_key=s" => \$snmp_priv_key
);

# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 3)
{
        if(!defined( $snmp_user ) || !defined( $snmp_auth_prot ) || !defined( $snmp_auth_key ) ){
                printf("UNKNOWN ERROR: Missing importants SNMPv3 parameters (user, auth_prot, auth_key)\n");
                exit $ERRORS{"UNKNOWN"};
        }
        if( $snmp_seclvl =~ /authNoPriv/i ){
                ($snmp, $snmperror) = Net::SNMP->session(
                        -hostname => $hostname,
                        -version => 'snmpv3',
                        -username => $snmp_user,
                        -authprotocol => lc $snmp_auth_prot,
                        -authpassword => $snmp_auth_key,
                        -translate   => [ -timeticks => 0x0 ]  # Turn off so sysUpTime is numeric]
                );
        }
        elsif( $snmp_seclvl =~ /authPriv/i ){
                if(!defined( $snmp_priv_prot ) || !defined( $snmp_priv_key ) ){
                        printf("UNKNOWN ERROR: Missing importants authPriv SNMPv3 parameters (priv_prot, priv_key)\n");
                        exit $ERRORS{"UNKNOWN"};
                }
                ($snmp, $snmperror) = Net::SNMP->session(
                        -hostname => $hostname,
                        -version => 'snmpv3',
                        -username => $snmp_user,
                        -authprotocol => lc $snmp_auth_prot,
                        -authpassword => $snmp_auth_key,
                        -privprotocol => lc $snmp_priv_prot,
                        -privpassword => $snmp_priv_key,
                        -translate   => [ -timeticks => 0x0 ]  # Turn off so sysUpTime is numeric]
                );
        }
        else{
                printf("UNKNOWN ERROR: Unknown security level for SNMPv3\n");
                exit $ERRORS{"UNKNOWN"};
        }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community, 
		-translate   => [ -timeticks => 0x0 ]  # Turn off so sysUpTime is numeric]
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community
		-translate   => [ -timeticks => 0x0 ]
	);
}

if($snmperror ne ""){
        printf($snmperror."\n");
        exit $ERRORS{"UNKNOWN"};
}

my $interface_description = "1.3.6.1.2.1.2.2.1.2";
my $interface_name = "1.3.6.1.2.1.31.1.1.1.1";
my $interface_index = "1.3.6.1.2.1.2.2.1.1";
my $interface_operstatus ="1.3.6.1.2.1.2.2.1.8";
my $interface_InOctets="1.3.6.1.2.1.2.2.1.10";
my $interface_OutOctets="1.3.6.1.2.1.2.2.1.16";
my $interface_InErrors="1.3.6.1.2.1.2.2.1.14";
my $interface_OutErrors="1.3.6.1.2.1.2.2.1.20";
my $interface_mtu="1.3.6.1.2.1.2.2.1.4";
my $interface_speed="1.3.6.1.2.1.2.2.1.5";
my $interface_lastChange="1.3.6.1.2.1.2.2.1.9";
my $sys_uptime="1.3.6.1.2.1.1.3.0";
my $interface_alias="1.3.6.1.2.1.31.1.1.1.18";
my $ip_ifindex ="1.3.6.1.2.1.4.20.1.2";
my %ifoperstatus = (
		1 => "up(1)",
		2 => "down(2)",
		3 => "testing(3)"
);
my $protocol=GesaProtocol->new();
if($type eq 'switch')
{
	my $resultat = $snmp->get_table(Baseoid => $interface_index);
	if (!defined($resultat)) 
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
				-hostname => $hostname,
				-version => 'snmpv1',
				-community => $community
			);
			$resultat = $snmp->get_table(Baseoid => $interface_index);
			if (!defined($resultat)) 
			{
		   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		   		$snmp->close;
		  		 exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}

	my @array;
	foreach my $key ( keys %$resultat) 
	{
		push @array,$$resultat{$key};
	}
	@array = sort par_num @array;
	foreach my $ifindex ( @array ) 
	{
		my $result = $snmp->get_request("$interface_description.$ifindex");
		my  $description =  $$result{"$interface_description.$ifindex"};
		$result = $snmp->get_request("$interface_operstatus.$ifindex");
		my  $status =  $$result{"$interface_operstatus.$ifindex"};
		$result = $snmp->get_request("$interface_speed.$ifindex");
		my  $speed =  $$result{"$interface_speed.$ifindex"};
		print $ifindex."|".$description."|".$status."|".$speed."\n";			
	}
}
elsif($stat!=0)
{
	my $ifindex = $stat;
	my $result = $snmp->get_request("$interface_description.$ifindex");
	if (!defined($result)) {
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
				-hostname => $hostname,
				-version => 'snmpv1',
				-community => $community
			);
			$result = $snmp->get_request("$interface_description.$ifindex");
			if (!defined($result)) 
			{
		   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		   		$snmp->close;
		  		 exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	my  $description =  $$result{"$interface_description.$ifindex"};

	$result = $snmp->get_request("$interface_operstatus.$ifindex");
	my  $status =  $$result{"$interface_operstatus.$ifindex"};

	$result = $snmp->get_request("$interface_name.$ifindex");
	my  $name =  $$result{"$interface_name.$ifindex"};

	$result = $snmp->get_request("$interface_InOctets.$ifindex");
	my  $InOctets =  $$result{"$interface_InOctets.$ifindex"};

	$result = $snmp->get_request("$interface_OutOctets.$ifindex");
	my  $OutOctets =  $$result{"$interface_OutOctets.$ifindex"};

	$result = $snmp->get_request("$interface_InErrors.$ifindex");
	my  $InErrors =  $$result{"$interface_InErrors.$ifindex"};

	$result = $snmp->get_request("$interface_OutErrors.$ifindex");
	my  $OutErrors =  $$result{"$interface_OutErrors.$ifindex"};

	$result = $snmp->get_request("$interface_mtu.$ifindex");
	my  $mtu =  $$result{"$interface_mtu.$ifindex"};

	$result = $snmp->get_request("$interface_speed.$ifindex");
	my  $speed =  $$result{"$interface_speed.$ifindex"};

	$result = $snmp->get_request("$interface_lastChange.$ifindex");
	my  $uptime =  $$result{"$interface_lastChange.$ifindex"};

	$result = $snmp->get_request("$interface_alias.$ifindex");
	my  $alias =  $$result{"$interface_alias.$ifindex"};
	$protocol->addLine("$ifindex","$description","$name","$status","$InOctets","$OutOctets","$InErrors","$OutErrors","$mtu","$speed","$uptime","$alias");
	#print $ifindex."|".$description."|".$name."|".$status."|".$InOctets."|".$OutOctets."|".$InErrors."|".$OutErrors."|".$mtu."|".$speed."|".$uptime."|".$alias."\n";
}
else
{
	my $ip_tables = $snmp->get_table(Baseoid => $ip_ifindex);
	if (!defined($ip_tables)) {
	   printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   $snmp->close;
	   exit $ERRORS{"UNKNOWN"};
	}

	my $resultat = $snmp->get_table(Baseoid => $interface_index);
	my @array;
	foreach my $key ( keys %$resultat) 
	{
		push @array,$$resultat{$key};
	}
	@array = sort par_num @array;

	foreach my $ifindex ( @array ) 
	{
		#my $ifindex = $$resultat{$key};
		my $result = $snmp->get_request("$interface_name.$ifindex");
		my $name =  $$result{"$interface_name.$ifindex"};
		$result = $snmp->get_request("$interface_description.$ifindex");
		my  $description =  $$result{"$interface_description.$ifindex"};

		#$result = $snmp->get_request("$interface_operstatus.$ifindex");
		#my  $operstatus =  $$result{"$interface_operstatus.$ifindex"};
		my $ip='';
		foreach my $ip_key ( keys %$ip_tables)
		{
			if($$ip_tables{$ip_key}==$ifindex)
			{
				if($ip_key=~/([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})$/)
				{
					$ip=$1;
				}
			}
		}
		#print "$ifindex|$name|$ip|\n";
		#$description =  pack("H*",$description);
		#$description =~ s/(.)/sprintf("%X",ord($1))/eg;

		chop($description);
		print $ifindex."|".$name."|".$description."|".$ip."\n";
	}
}
$protocol->print();
$snmp->close();
exit(0);




