#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Std;
use Text::Trim;

use FindBin;
use lib $FindBin::Bin.'/lib';
use GesaProtocol;


my $delimiter="§";
my $protocol=GesaProtocol->new(4000,"\r\n");

my %columns=(
        "services"      => "Caption,Description,PathName,ServiceType,Started,StartMode,State,Status",
        "disk_drive"          => "CapabilityDescriptions,Caption,Description,DeviceID,FirmwareRevision,Index,Manufacturer,MediaType,Model,Name,Partitions,SerialNumber,Size,Status",
	"disk_physical"	=> "*",
	"disk_logical" 	=> "*",
	"disk_part"	=> "*",
        "os"            => "Name,CSDVersion,InstallDate,LastBootUpTime,LocalDateTime,MUILanguages,NumberOfLicensedUsers,NumberOfProcesses,NumberOfUsers,OSArchitecture,RegisteredUser,SerialNumber,Status",
        "net_perf"      => "CurrentBandwidth,BytesReceivedPerSec,BytesSentPerSec,Name,Frequency_Sys100NS,OutputQueueLength,PacketsReceivedErrors,PacketsReceivedPerSec,PacketsSentPerSec,Timestamp_Sys100NS",
        "net_adapter"   => "macaddress,netconnectionID",
        "net_config"    => "dnsdomain,dnshostname,ipaddress,ipsubnet,macaddress,description,defaultipgateway,dhcpenabled,dhcpserver,servicename",
#       "log"           => "EventCode,EventIdentifier,Type,LogFile,SourceName,Message,TimeGenerated",
        "printer"       => "MarkingTechnology,default",
        "software"      => "Caption,Description,InstallDate,InstallLocation,InstallState,InstallSource,Name,PackageName,Vendor,Version",
        "features"      => "*",
        "cpu"           => "Name",
	"mb"		=> "Manufacturer,Product,Version",
        "updates"       => "Caption,Description,HotFixID,InstalledOn",
        "spyware"       => "*",
	"antivirus"	=> "*",
	"firewall"	=> "*",
);

my %tables=(
        "services"      => "Win32_Service",
        "disk_drive"    => "Win32_DiskDrive",
	"disk_physical" => "Win32_PhysicalDisk",
	"disk_logical"  => "Win32_LogicalDisk",
	"disk_part"	=> "Win32_DiskPartition",
        "os"            => "Win32_OperatingSystem",
        "net_perf"      => "Win32_PerfRawData_Tcpip_NetworkInterface",
        "net_adapter"   => "win32_networkadapter",
        "net_config"    => "Win32_NetworkAdapterConfiguration",
#       "log"           => "Win32_NTLogEvent",
        "printer"       => "Win32_Printer",
        "software"      => "Win32_Product",
        "feature"       => "Win32_OptionalFeature",
        "cpu"           => "Win32_Processor",	
	"mb"		=> "Win32_BaseBoard",
        "updates"       => "Win32_quickfixengineering",
        "spyware"       => "AntiSpywareProduct",
	"antivirus"	=> "AntiVirusProduct",
	"firewall"	=> "FirewallProduct",
);

my %namespaces=(
        "spyware"       => 'root\securitycenter2',
	"antivirus"	=> 'root\securitycenter2',
        "firewall"     => 'root\securitycenter2',
);


my %ERRORS=( 
        "OK"            => 0,
        "WARNING"       => 1,
        "CRITICAL"      => 2,
        "UNKNOWN"       => 3,
);

my $wmic_command="wmic -A  /etc/esia/wmi.conf --delimiter=$delimiter";

my $MAX = 4000;

sub error_exit_unknown{
	my ($line) = @_;
	if($line =~ m/(ERROR:.*)/)	{
		print $1 . "\n";
		exit $ERRORS{UNKNOWN};
	}
}

my %opts;

sub prepare_request{
	my ( $mode, $filterName, $filterRegex ) = @_;

	my $table=$tables{$mode};
	my $column=$columns{$mode};

	error_exit_unknown("ERROR: Table not defined") unless defined $table;
	$column="*" 				unless defined $column;
	
	my $req="select $column from $table";

	if( defined $filterName ) {
		error_exit_unknown ("ERROR: filterName defined without filterRegex") unless defined $filterRegex;
		$req = $req . ' where '. $filterName . ' like "'. $filterRegex . '"';
	}	
	return $req;	
}

sub selectColumns{
	my ( $cl, @data ) = @_;
	my @ret;

	return @data unless ( defined $cl && lc($cl) ne "null");

	foreach my $v (split(/,/,$cl)){
		push @ret,$data[$v];
	}
	return @ret;
}

sub send_request{
	my ( $ip, $req, $mode, $min, $max, $cl ) = @_;
	my @output;
	my $namespace=$namespaces{$mode};
	
	error_exit_unknown("ERROR: No Host defined")		unless defined $ip;
	error_exit_unknown("ERROR: No request defined")		unless defined $req;

	$min = 0 						unless defined $min;
	$max = $MAX - 2						unless defined $max;

	error_exit_unknown('ERROR: Min parameter must be one Integer')	if ($min !~  m/^\d+$/) ;
        error_exit_unknown('ERROR: Max parameter must be one Integer')	if ($max !~  m/^\d+$/) ;
 
	my $command=$wmic_command;
	$command = $command. " --namespace='" . $namespace . "' " if defined $namespace;
	$command=$command . " //" . $ip  . " '" . $req . "' 2>&1";

	my $i=0;

	my $idx="nl";
        my $firstLine="<EMPTY>";
	my $ret;

	open(my $results,$command . "|");
	while( my $line = <$results> ){
                error_exit_unknown($line);

		return @output if ( $i > $max ); 
	
		if( $line !~ m/CLASS:/ && $firstLine ne $line ){
                        $firstLine=$line if $i==0;


			$idx=$i	if $i>0;
                        $line="$idx$delimiter$line";

			chomp $line;
			$ret=$protocol->addLine(selectColumns($cl,split(/$delimiter/,$line)))	if $i >= $min ;
			$ret and last;
	
			$i++;
		}	
	}
        close $results;
	return ;
}

getopt("H:m:l:c:f:", \%opts);

error_exit_unknown("ERROR: Parameter Mode not defined") unless defined $opts{m};

my @lim;
@lim = split(/,/,$opts{l})	if defined $opts{l};
$lim[1]=$MAX - 2		unless defined $lim[1];

my $request;
if ( defined $opts{f} ){
	my @ff=split(/=/,$opts{f});
	error_exit_unknown("ERROR: The Filter param must have the form <name>=<filter>") unless scalar @ff == 2;	
        $request=prepare_request($opts{m},$ff[0],$ff[1]);
}
else {
	$request=prepare_request($opts{m});
}

send_request($opts{H},$request,$opts{m},$lim[0],$lim[1],$opts{c});
$protocol->print();
